from pydantic import BaseModel, Field


class AbstractChargerState(BaseModel):
    current_power: float = Field(ge=0, le=10000)


class AbstractCharger(BaseModel):
    name: str = Field(min_length=3, max_length=13, frozen=True)
    max_power: float = Field(ge=0, le=10000, frozen=True)
    max_power_to_car: float = Field(ge=0, le=10000)
    current_state: AbstractChargerState



from pydantic import BaseModel, Field


class AbstractState(BaseModel):
    pass


class AbstractBatteryState(AbstractState):
    soc: float = Field(ge=0, le=1)
    current_power: float = Field(ge=0)

    def __new__(cls, *args, **kwargs):
        if cls is AbstractBatteryState:
            raise Exception("cannot instantiate a instance of abstract class")
        return super().__new__(cls)


class AbstractBattery(BaseModel):
    name: str = Field(max_length=30, min_length=3, frozen=True)
    nameplate_capacity: float = Field(ge=0, le=10000, frozen=True)
    capacity: float = Field(ge=0, le=10000)
    inverter_size: float = Field(ge=0, le=10000, frozen=True)
    current_state: AbstractBatteryState

    def __new__(cls, *args, **kwargs):
        if cls is AbstractBattery:
            raise Exception("Cannot instantiate an instance of abstract class")
        return super().__new__(cls)


from ev_station_assets.abstract.battery import AbstractBattery, AbstractBatteryState
from ev_station_assets.abstract.ev_chargers import AbstractCharger, AbstractChargerState
from ev_station_assets.abstract.onsite_loads import AbstractSiteLoad, AbstractSiteLoadState
from ev_station_assets.abstract.generators import AbstractGenerator, AbstractGeneratorState


class GenericBatteryState(AbstractBatteryState):
    pass


class GenericBattery(AbstractBattery):
    pass


class GenericEVChargerState(AbstractChargerState):
    pass


class GenericEVCharger(AbstractCharger):
    pass


class GenericSiteLoadState(AbstractSiteLoadState):
    pass


class GenericSiteLoad(AbstractSiteLoad):
    pass


class GenericGeneratorState(AbstractGeneratorState):
    pass


class GenericGenerator(AbstractGenerator):
    pass



from pydantic import BaseModel, Field


class AbstractGeneratorState(BaseModel):
    power: float = Field(ge=0.00)
    fuel_state: float = Field(ge=0.00, le=1.00)


class AbstractGenerator(BaseModel):
    name: str = Field(min_length=3, max_length=30, frozen=True)
    efficiency: float = Field(ge=0.00, le=1.00)
    input_cost_per_unit_fuel: float = Field(ge=0.00)
    fuel_capacity_units: float = Field(ge=0.00)
    current_state: AbstractGeneratorState



from pydantic import BaseModel, Field


class AbstractSiteLoadState(BaseModel):
    current_load: float = Field(ge=0.00)


class AbstractSiteLoad(BaseModel):
    name: str = Field(min_length=3, max_length=30, frozen=True)
    max_load: float = Field(ge=0.00, frozen=True)
    current_state: AbstractSiteLoadState
